/*
 * MP3 Header reader
 *
 * Copyright (c) 2006 vixy project
 *
 * This file contains the code that based on FFmpeg (http://ffmpeg.mplayerhq.hu/)
 * See original copyright notice in /FFMPEG_CREDITS and /FFMPEG_IMPORTS
 *
 * This file is part of VIXY FLV Converter.
 *
 * 'VIXY FLV Converter' is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * 'VIXY FLV Converter' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "mp3header.h"

static const uint16 mpa_bitrate_tab[2][3][15] = {
    { {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448 },
      {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384 },
      {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320 } },
    { {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256},
      {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160},
      {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
    }
};

static const uint16 mpa_freq_tab[3] = { 44100, 48000, 32000 };

#define MPA_STEREO  0
#define MPA_JSTEREO 1
#define MPA_DUAL    2
#define MPA_MONO    3

int decode_mp3_header(MP3HEADER*s, uint32 header)
{
    int frame_size, mpeg25;
    int sample_rate_index, bitrate_index;
    int lsf, layer, mode;
	
	if ((header & 0xfff00000) != 0xfff00000) return -1;
	
	if (header & (1<<20)) 
	{
        lsf = (header & (1<<19)) ? 0 : 1;
        mpeg25 = 0;
    } 
	else 
	{
        lsf = 1;
        mpeg25 = 1;
    }

    layer = 4 - ((header >> 17) & 3);
    if (layer != 3) return -1; // not mp3
	
    sample_rate_index = (header >> 10) & 3;
	s->sample_rate = mpa_freq_tab[sample_rate_index] >> (lsf + mpeg25);
    
//	error_protection = ((header >> 16) & 1) ^ 1;

    bitrate_index = (header >> 12) & 0xf;
    if (bitrate_index == 0) return -1;

//    padding = (header >> 9) & 1;
    mode = (header >> 6) & 3;
//    mode_ext = (header >> 4) & 3;

    if (mode == MPA_MONO)
        s->channels = 1;
    else
        s->channels = 2;
	
	frame_size = mpa_bitrate_tab[lsf][layer - 1][bitrate_index];
	s->bit_rate = frame_size * 1000;

	if (lsf)
		s->frame_size = 576;
	else
		s->frame_size = 1152;

	return 0;
}

